set(FW_BUILD_EXTERNAL 1)

include("${CMAKE_CURRENT_LIST_DIR}/SightRequirements.cmake")
include("${CMAKE_CURRENT_LIST_DIR}/macros.cmake")

set(_SUPPORTED_COMPONENTS fwCore;fwRuntime;fwThread;fwTools;fwCom;fwCamp;fwMemory;fwData;fwDataCamp;fwMath;fwMedData;fwActivities;fwServices;fwMedDataCamp;fwDataTools;fwLog;fwPreferences;fwGui;fwJobs;fwQt;fwGuiQt;fwMedDataTools;fwAtoms;fwAtomConversion;fwZip;fwAtomsBoostIO;fwAtomsPatch;fwAtomsFilter;fwMDSemanticPatch;fwIO;fwDataIO;fwItkIO;fwVtkIO;vtkGdcmIO;fwRender;fwRenderVTK;fwRenderQt;fwDicomIOFilter;fwDicomIOFilterQt;fwDicomTools;fwGdcmIO;fwPacsIO;fwNetworkIO;fwStructuralPatch)

if(NOT USE_SYSTEM_LIB)
    if(NOT FW_CONAN_CONFIGURED)
        include("${CMAKE_CURRENT_LIST_DIR}/conan-deps.cmake")
        include("${CMAKE_CURRENT_LIST_DIR}/conan.cmake")
        list(APPEND CMAKE_PREFIX_PATH "${Sight_DIR}/../../..")
        add_definitions(${CONAN_DEFINES})
        set(FW_CONAN_CONFIGURED 1)
    endif()
else()
    string(TOLOWER ${PRJ_NAME} APP_NAME)
    configure_file(${FWCMAKE_RESOURCE_PATH}/install/linux/template.sh.in ${CMAKE_CURRENT_BINARY_DIR}/${APP_NAME} @ONLY)
endif()

setExternalLibrariesDir()

set(FWBUNDLE_LIB_PREFIX lib/x86_64-linux-gnu/sight)
set(FWBUNDLE_RC_PREFIX share/sight)

file(TO_CMAKE_PATH "${Sight_DIR}/../../../bin" Sight_BINARY_DIR)
file(TO_CMAKE_PATH "${Sight_DIR}/../../../${FWBUNDLE_LIB_PREFIX}" Sight_LIBRARY_DIR)
file(TO_CMAKE_PATH "${Sight_DIR}/../../../${FWBUNDLE_RC_PREFIX}" Sight_BUNDLES_DIR)
file(TO_CMAKE_PATH "${Sight_DIR}/../../../" Sight_DEFINE_DIR)

foreach(_comp ${Sight_FIND_COMPONENTS})
    if(NOT ${_comp}_FOUND)
        message(STATUS "Looking for Sight component: ${_comp}")
        if (NOT ";${_SUPPORTED_COMPONENTS};" MATCHES ${_comp})
            set(Sight_FOUND False)
            set(Sight_NOT_FOUND_MESSAGE "Unsupported component: ${_comp}")
        endif()
        if(EXISTS "${CMAKE_CURRENT_LIST_DIR}/Sight-${_comp}Dependencies.cmake")
            include("${CMAKE_CURRENT_LIST_DIR}/Sight-${_comp}Dependencies.cmake")
        endif()

        include("${CMAKE_CURRENT_LIST_DIR}/Sight-${_comp}Targets.cmake")

        # For now I do not know how to unroll this loop when configuring the file... so for now just use the default log level
        set_property( TARGET Sight::${_comp} PROPERTY INTERFACE_COMPILE_DEFINITIONS SPYLOG_LEVEL=2 SIGHT_DIR="${Sight_DEFINE_DIR}")
        set(${_comp}_FOUND 1)
    endif()
endforeach()

set(SPYLOG_LEVEL "error" CACHE STRING "Log level")
set(SPYLOG_LEVEL_VALUES "fatal;error;warning;info;debug;trace" CACHE INTERNAL
    "List of possible values for log level")
set(SPYLOG_LEVEL_MAP_fatal 1)
set(SPYLOG_LEVEL_MAP_error 2)
set(SPYLOG_LEVEL_MAP_warning 3)
set(SPYLOG_LEVEL_MAP_info 4)
set(SPYLOG_LEVEL_MAP_debug 5)
set(SPYLOG_LEVEL_MAP_trace 6)
set_property(CACHE SPYLOG_LEVEL PROPERTY STRINGS ${SPYLOG_LEVEL_VALUES} )

